# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.collection import Collection
from ..types.response_type import ResponseType
from ..types.search_response import SearchResponse
from ..types.source_connection_job import SourceConnectionJob
from .raw_client import AsyncRawCollectionsClient, RawCollectionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CollectionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCollectionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCollectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCollectionsClient
        """
        return self._raw_client

    def list_collections(
        self,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Collection]:
        """
        List all collections for the current user's organization.

        Parameters
        ----------
        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Collection]
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
        )
        client.collections.list_collections()
        """
        _response = self._raw_client.list_collections(skip=skip, limit=limit, request_options=request_options)
        return _response.data

    def create_collection(
        self,
        *,
        name: str,
        readable_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Collection:
        """
        Create a new collection.

        Parameters
        ----------
        name : str
            Display name for the collection

        readable_id : typing.Optional[str]
            Unique lowercase identifier (e.g., respectable-sparrow, collection-123)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Collection
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
        )
        client.collections.create_collection(
            name="Finance Data",
            readable_id="finance-data",
        )
        """
        _response = self._raw_client.create_collection(
            name=name, readable_id=readable_id, request_options=request_options
        )
        return _response.data

    def get_collection(
        self, readable_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Collection:
        """
        Get a specific collection by its readable ID.

        Parameters
        ----------
        readable_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Collection
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
        )
        client.collections.get_collection(
            readable_id="readable_id",
        )
        """
        _response = self._raw_client.get_collection(readable_id, request_options=request_options)
        return _response.data

    def delete_collection(
        self,
        readable_id: str,
        *,
        delete_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Collection:
        """
        Delete a collection by its readable ID.

        Args:
            readable_id: The readable ID of the collection to delete
            delete_data: Whether to delete the data in destinations
            db: The database session
            current_user: The current user

        Returns:
            The deleted collection

        Parameters
        ----------
        readable_id : str

        delete_data : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Collection
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
        )
        client.collections.delete_collection(
            readable_id="readable_id",
        )
        """
        _response = self._raw_client.delete_collection(
            readable_id, delete_data=delete_data, request_options=request_options
        )
        return _response.data

    def search_collection(
        self,
        readable_id: str,
        *,
        query: str,
        response_type: typing.Optional[ResponseType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchResponse:
        """
        Search within a collection identified by readable ID.

        Args:
            readable_id: The readable ID of the collection to search
            query: The search query
            response_type: Type of response (raw results or AI completion)
            db: The database session
            current_user: The current user

        Returns:
            dict: Search results or AI completion response

        Parameters
        ----------
        readable_id : str

        query : str
            Search query

        response_type : typing.Optional[ResponseType]
            Type of response: raw search results or AI completion

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchResponse
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
        )
        client.collections.search_collection(
            readable_id="readable_id",
            query="query",
        )
        """
        _response = self._raw_client.search_collection(
            readable_id, query=query, response_type=response_type, request_options=request_options
        )
        return _response.data

    def refresh_all_source_connections(
        self, readable_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[SourceConnectionJob]:
        """
        Start sync jobs for all source connections in the collection.

        Args:
            readable_id: The readable ID of the collection
            db: The database session
            current_user: The current user
            background_tasks: Background tasks for async operations

        Returns:
            A list of created sync jobs

        Parameters
        ----------
        readable_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceConnectionJob]
            Successful Response

        Examples
        --------
        from airweave import AirweaveSDK

        client = AirweaveSDK(
            api_key="YOUR_API_KEY",
        )
        client.collections.refresh_all_source_connections(
            readable_id="readable_id",
        )
        """
        _response = self._raw_client.refresh_all_source_connections(readable_id, request_options=request_options)
        return _response.data


class AsyncCollectionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCollectionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCollectionsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCollectionsClient
        """
        return self._raw_client

    async def list_collections(
        self,
        *,
        skip: typing.Optional[int] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Collection]:
        """
        List all collections for the current user's organization.

        Parameters
        ----------
        skip : typing.Optional[int]

        limit : typing.Optional[int]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Collection]
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.collections.list_collections()


        asyncio.run(main())
        """
        _response = await self._raw_client.list_collections(skip=skip, limit=limit, request_options=request_options)
        return _response.data

    async def create_collection(
        self,
        *,
        name: str,
        readable_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Collection:
        """
        Create a new collection.

        Parameters
        ----------
        name : str
            Display name for the collection

        readable_id : typing.Optional[str]
            Unique lowercase identifier (e.g., respectable-sparrow, collection-123)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Collection
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.collections.create_collection(
                name="Finance Data",
                readable_id="finance-data",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create_collection(
            name=name, readable_id=readable_id, request_options=request_options
        )
        return _response.data

    async def get_collection(
        self, readable_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> Collection:
        """
        Get a specific collection by its readable ID.

        Parameters
        ----------
        readable_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Collection
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.collections.get_collection(
                readable_id="readable_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_collection(readable_id, request_options=request_options)
        return _response.data

    async def delete_collection(
        self,
        readable_id: str,
        *,
        delete_data: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Collection:
        """
        Delete a collection by its readable ID.

        Args:
            readable_id: The readable ID of the collection to delete
            delete_data: Whether to delete the data in destinations
            db: The database session
            current_user: The current user

        Returns:
            The deleted collection

        Parameters
        ----------
        readable_id : str

        delete_data : typing.Optional[bool]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Collection
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.collections.delete_collection(
                readable_id="readable_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_collection(
            readable_id, delete_data=delete_data, request_options=request_options
        )
        return _response.data

    async def search_collection(
        self,
        readable_id: str,
        *,
        query: str,
        response_type: typing.Optional[ResponseType] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchResponse:
        """
        Search within a collection identified by readable ID.

        Args:
            readable_id: The readable ID of the collection to search
            query: The search query
            response_type: Type of response (raw results or AI completion)
            db: The database session
            current_user: The current user

        Returns:
            dict: Search results or AI completion response

        Parameters
        ----------
        readable_id : str

        query : str
            Search query

        response_type : typing.Optional[ResponseType]
            Type of response: raw search results or AI completion

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchResponse
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.collections.search_collection(
                readable_id="readable_id",
                query="query",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.search_collection(
            readable_id, query=query, response_type=response_type, request_options=request_options
        )
        return _response.data

    async def refresh_all_source_connections(
        self, readable_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[SourceConnectionJob]:
        """
        Start sync jobs for all source connections in the collection.

        Args:
            readable_id: The readable ID of the collection
            db: The database session
            current_user: The current user
            background_tasks: Background tasks for async operations

        Returns:
            A list of created sync jobs

        Parameters
        ----------
        readable_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[SourceConnectionJob]
            Successful Response

        Examples
        --------
        import asyncio

        from airweave import AsyncAirweaveSDK

        client = AsyncAirweaveSDK(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.collections.refresh_all_source_connections(
                readable_id="readable_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.refresh_all_source_connections(readable_id, request_options=request_options)
        return _response.data
